@@ -23,46 +23,50 @@ require 'mcollective/monkey_patches'
 #   http://www.devco.net/archives/2009/10/18/middleware_for_systems_administration.php
 module MCollective
   # Exceptions for the RPC system
+  class DDLValidationError<RuntimeError;end
+  class MsgDoesNotMatchRequestID < RuntimeError; end
+  class MsgTTLExpired<RuntimeError;end
+  class NotTargettedAtUs<RuntimeError;end
   class RPCError<StandardError;end
+  class SecurityValidationFailed<RuntimeError;end
+
+  class InvalidRPCData<RPCError;end
+  class MissingRPCData<RPCError;end
   class RPCAborted<RPCError;end
   class UnknownRPCAction<RPCError;end
-  class MissingRPCData<RPCError;end
-  class InvalidRPCData<RPCError;end
   class UnknownRPCError<RPCError;end
-  class NotTargettedAtUs<RuntimeError;end
-  class SecurityValidationFailed<RuntimeError;end
-  class DDLValidationError<RuntimeError;end
-  class MsgTTLExpired<RuntimeError;end
-  class MsgDoesNotMatchRequestID < RuntimeError; end
 
-
-  autoload :Config, "mcollective/config"
-  autoload :Log, "mcollective/log"
-  autoload :Logger, "mcollective/logger"
-  autoload :Runner, "mcollective/runner"
-  autoload :RunnerStats, "mcollective/runnerstats"
   autoload :Agents, "mcollective/agents"
-  autoload :Client, "mcollective/client"
-  autoload :Util, "mcollective/util"
-  autoload :Optionparser, "mcollective/optionparser"
-  autoload :Connector, "mcollective/connector"
-  autoload :Security, "mcollective/security"
-  autoload :Facts, "mcollective/facts"
-  autoload :Registration, "mcollective/registration"
-  autoload :PluginManager, "mcollective/pluginmanager"
-  autoload :RPC, "mcollective/rpc"
-  autoload :Matcher, "mcollective/matcher"
-  autoload :Message, "mcollective/message"
-  autoload :SSL, "mcollective/ssl"
   autoload :Application, "mcollective/application"
   autoload :Applications, "mcollective/applications"
-  autoload :Vendor, "mcollective/vendor"
-  autoload :Shell, "mcollective/shell"
+  autoload :Client, "mcollective/client"
+  autoload :Config, "mcollective/config"
+  autoload :Connector, "mcollective/connector"
+  autoload :Data, "mcollective/data"
+  autoload :DDL, "mcollective/ddl"
+  autoload :Discovery, "mcollective/discovery"
+  autoload :Facts, "mcollective/facts"
+  autoload :Logger, "mcollective/logger"
+  autoload :Log, "mcollective/log"
+  autoload :Matcher, "mcollective/matcher"
+  autoload :Message, "mcollective/message"
+  autoload :Optionparser, "mcollective/optionparser"
+  autoload :Generators, "mcollective/generators"
+  autoload :PluginManager, "mcollective/pluginmanager"
   autoload :PluginPackager, "mcollective/pluginpackager"
+  autoload :Registration, "mcollective/registration"
+  autoload :RPC, "mcollective/rpc"
+  autoload :Runner, "mcollective/runner"
+  autoload :RunnerStats, "mcollective/runnerstats"
+  autoload :Security, "mcollective/security"
+  autoload :Shell, "mcollective/shell"
+  autoload :SSL, "mcollective/ssl"
+  autoload :Util, "mcollective/util"
+  autoload :Vendor, "mcollective/vendor"
 
   MCollective::Vendor.load_vendored
 
-  VERSION="2.0.0"
+  VERSION="2.1.0"
 
   def self.version
     VERSION
