@@ -171,12 +171,12 @@ module MCollective
           end
 
           # type was given and its not one of our special types, just pass it onto optparse
-          opts_array << carg[:type] if carg[:type] and ! [:bool, :array].include?(carg[:type])
+          opts_array << carg[:type] if carg[:type] && ![:boolean, :bool, :array].include?(carg[:type])
 
           opts_array << carg[:description]
 
           # Handle our special types else just rely on the optparser to handle the types
-          if carg[:type] == :bool
+          if [:bool, :boolean].include?(carg[:type])
             parser.send(*opts_array) do |v|
               validate_option(carg[:validate], carg[:name], v)
 
@@ -255,7 +255,11 @@ module MCollective
         raise(e)
       end
 
-      err_dest.puts "#{$0} failed to run: #{e} (#{e.class})"
+      if options.nil? || options[:verbose]
+        err_dest.puts "\n%s %s" % [ Util.colorize(:red, e.to_s), Util.colorize(:bold, "(#{e.class.to_s})")]
+      else
+        err_dest.puts "\nThe %s application failed to run, use -v for full error details: %s\n" % [ Util.colorize(:bold, $0), Util.colorize(:red, e.to_s)]
+      end
 
       if options.nil? || options[:verbose]
         e.backtrace.each{|l| err_dest.puts "\tfrom #{l}"}
@@ -307,7 +311,7 @@ module MCollective
     # Exit with 0 if no discovery were done and > 0 responses were received
     # Exit with 1 if no nodes were discovered
     # Exit with 2 if nodes were discovered but some RPC requests failed
-    # Exit with 3 if nodes were discovered, but not responses receivedif
+    # Exit with 3 if nodes were discovered, but not responses received
     # Exit with 4 if no discovery were done and no responses were received
     def halt(stats)
       request_stats = {:discoverytime => 0,
