@@ -1,7 +1,7 @@
 module MCollective
   # Helpers for writing clients that can talk to agents, do discovery and so forth
   class Client
-    attr_accessor :options, :stats
+    attr_accessor :options, :stats, :discoverer
 
     def initialize(configfile)
       @config = Config.instance
@@ -14,6 +14,7 @@ module MCollective
       @options = nil
       @subscriptions = {}
 
+      @discoverer = Discovery.new(self)
       @connection.connect
     end
 
@@ -114,28 +115,10 @@ module MCollective
     def discover(filter, timeout, limit=0)
       raise "Limit has to be an integer" unless limit.is_a?(Fixnum)
 
-      begin
-        hosts = []
-        Timeout.timeout(timeout) do
-          reqid = sendreq("ping", "discovery", filter)
-          Log.debug("Waiting #{timeout} seconds for discovery replies to request #{reqid}")
+      compount_timeout = timeout_for_compound_filter(@options[:filter]["compound"])
+      timeout = timeout + compount_timeout
 
-          loop do
-            reply = receive(reqid)
-            Log.debug("Got discovery reply from #{reply.payload[:senderid]}")
-            hosts << reply.payload[:senderid]
-
-            return hosts if limit > 0 && hosts.size == limit
-          end
-        end
-      rescue Timeout::Error => e
-      rescue Exception => e
-        raise
-      ensure
-        unsubscribe("discovery", :reply)
-      end
-
-      hosts.sort
+      discovered = @discoverer.discover(filter, timeout, limit)
     end
 
     # Send a request, performs the passed block for each response
@@ -157,12 +140,15 @@ module MCollective
 
       options = @options unless options
 
+      compount_timeout = timeout_for_compound_filter(options[:filter]["compound"])
+      timeout = options[:timeout] + compount_timeout
+
       STDOUT.sync = true
 
       hosts_responded = 0
 
       begin
-        Timeout.timeout(options[:timeout]) do
+        Timeout.timeout(timeout) do
           reqid = sendreq(body, agent, options[:filter])
 
           loop do
@@ -203,6 +189,9 @@ module MCollective
 
       options = @options unless options
 
+      compount_timeout = timeout_for_compound_filter(options[:filter]["compound"])
+      timeout = options[:timeout] + compount_timeout
+
       STDOUT.sync = true
 
       print("Determining the amount of hosts matching filter for #{options[:disctimeout]} seconds .... ")
@@ -224,7 +213,7 @@ module MCollective
       raise("No matching clients found") if discovered == 0
 
       begin
-        Timeout.timeout(options[:timeout]) do
+        Timeout.timeout(timeout) do
           reqid = sendreq(body, agent, options[:filter])
 
           (1..discovered).each do |c|
@@ -251,6 +240,27 @@ module MCollective
       return stat
     end
 
+    # if a compound filter is specified and it has any function
+    # then we read the DDL for each of those plugins and sum up
+    # the timeout declared in the DDL
+    def timeout_for_compound_filter(compound_filter)
+      return 0 if compound_filter.nil? || compound_filter.empty?
+
+      timeout = 0
+
+      compound_filter.each do |filter|
+        filter.each do |statement|
+          if statement["fstatement"]
+            pluginname = Data.pluginname(statement["fstatement"]["name"])
+            ddl = DDL.new(pluginname, :data)
+            timeout += ddl.meta[:timeout]
+          end
+        end
+      end
+
+      timeout
+    end
+
     # Prints out the stats returns from req and discovered_req in a nice way
     def display_stats(stats, options=false, caption="stomp call summary")
       options = @options unless options
