@@ -3,8 +3,9 @@ module MCollective
     # The main component of the Simple RPC client system, this wraps around MCollective::Client
     # and just brings in a lot of convention and standard approached.
     class Client
-      attr_accessor :discovery_timeout, :timeout, :verbose, :filter, :config, :progress, :ttl, :reply_to
+      attr_accessor :timeout, :verbose, :filter, :config, :progress, :ttl, :reply_to
       attr_reader :client, :stats, :ddl, :agent, :limit_targets, :limit_method, :output_format, :batch_size, :batch_sleep_time, :batch_mode
+      attr_reader :discovery_options, :discovery_method
 
       @@initial_options = nil
 
@@ -37,10 +38,10 @@ module MCollective
           @@initial_options = Marshal.dump(initial_options)
         end
 
+        @initial_options = initial_options
         @stats = Stats.new
         @agent = agent
-        @discovery_timeout = initial_options[:disctimeout]
-        @timeout = initial_options[:timeout]
+        @timeout = initial_options[:timeout] || 5
         @verbose = initial_options[:verbose]
         @filter = initial_options[:filter]
         @config = initial_options[:config]
@@ -51,6 +52,8 @@ module MCollective
         @output_format = initial_options[:output_format] || :console
         @force_direct_request = false
         @reply_to = initial_options[:reply_to]
+        @discovery_method = initial_options[:discovery_method]
+        @discovery_options = initial_options[:discovery_options] || []
 
         @batch_size = Integer(initial_options[:batch_size] || 0)
         @batch_sleep_time = Float(initial_options[:batch_sleep_time] || 1)
@@ -61,6 +64,8 @@ module MCollective
         @client = MCollective::Client.new(@config)
         @client.options = initial_options
 
+        @discovery_timeout = discovery_timeout
+
         @collective = @client.collective
         @ttl = initial_options[:ttl] || Config.instance.ttl
 
@@ -215,7 +220,7 @@ module MCollective
 
         @stats.reset
 
-        @ddl.validate_request(action, args) if @ddl
+        @ddl.validate_rpc_request(action, args) if @ddl
 
         # if a global batch size is set just use that else set it
         # in the case that it was passed as an argument
@@ -272,7 +277,7 @@ module MCollective
       # request which technically does not need filters.  If you try to use this
       # mode with direct addressing disabled an exception will be raise
       def custom_request(action, args, expected_agents, filter = {}, &block)
-        @ddl.validate_request(action, args) if @ddl
+        @ddl.validate_rpc_request(action, args) if @ddl
 
         if filter == {} && !Config.instance.direct_addressing
           raise "Attempted to do a filterless custom_request without direct_addressing enabled, preventing unexpected call to all nodes"
@@ -319,6 +324,30 @@ module MCollective
         end
       end
 
+      def discovery_timeout
+        return @initial_options[:disctimeout] if @initial_options[:disctimeout]
+        return @client.discoverer.ddl.meta[:timeout]
+      end
+
+      def discovery_method=(method)
+        @discovery_method = method
+
+        if @initial_options[:discovery_options]
+          @discovery_options = @initial_options[:discovery_options]
+        else
+          @discovery_options.clear
+        end
+
+        @client.options = options
+        @discovery_timeout = discovery_timeout
+        reset
+      end
+
+      def discovery_options=(options)
+        @discovery_options = [options].flatten
+        reset
+      end
+
       # Sets the class filter
       def class_filter(klass)
         @filter["cf_class"] << klass
@@ -359,7 +388,7 @@ module MCollective
 
       # Set a compound filter
       def compound_filter(filter)
-        @filter["compound"] << Matcher::Parser.new(filter).execution_stack
+        @filter["compound"] <<  Matcher.create_compound_callstack(filter)
         reset
       end
 
@@ -446,21 +475,36 @@ module MCollective
         unless @discovered_agents
           @stats.time_discovery :start
 
-          @stderr.print("Determining the amount of hosts matching filter for #{discovery_timeout} seconds .... ") if verbose
+          # if compound filters are used the only real option is to use the mc
+          # discovery plugin since its the only capable of using data queries etc
+          # and we do not want to degrade that experience just to allow compounds
+          # on other discovery plugins the UX would be too bad raising complex sets
+          # of errors etc.
+          @client.discoverer.force_discovery_method_by_filter(options[:filter])
+
+          actual_timeout = options[:disctimeout] + @client.timeout_for_compound_filter(options[:filter]["compound"])
+          if actual_timeout > 0
+            @stderr.print("Discovering hosts using the %s method for %d second(s) .... " % [@client.discoverer.discovery_method, actual_timeout]) if verbose
+          else
+            @stderr.print("Discovering hosts using the %s method .... " % [@client.discoverer.discovery_method]) if verbose
+          end
+
+          @client.options = options
 
           # if the requested limit is a pure number and not a percent
           # and if we're configured to use the first found hosts as the
           # limit method then pass in the limit thus minimizing the amount
           # of work we do in the discover phase and speeding it up significantly
           if @limit_method == :first and @limit_targets.is_a?(Fixnum)
-            @discovered_agents = @client.discover(@filter, @discovery_timeout, @limit_targets)
+            @discovered_agents = @client.discover(@filter, options[:disctimeout], @limit_targets)
           else
-            @discovered_agents = @client.discover(@filter, @discovery_timeout)
+            @discovered_agents = @client.discover(@filter, options[:disctimeout])
           end
 
-          @force_direct_request = false
           @stderr.puts(@discovered_agents.size) if verbose
 
+          @force_direct_request = @client.discoverer.force_direct_mode?
+
           @stats.time_discovery :end
         end
 
@@ -480,13 +524,18 @@ module MCollective
          :collective => @collective,
          :output_format => @output_format,
          :ttl => @ttl,
+         :discovery_method => @discovery_method,
+         :discovery_options => @discovery_options,
          :config => @config}
       end
 
       # Sets the collective we are communicating with
       def collective=(c)
+        raise "Unknown collective #{c}" unless Config.instance.collectives.include?(c)
+
         @collective = c
-        @client.options[:collective] = c
+        @client.options = options
+        reset
       end
 
       # Sets and sanity checks the limit_targets variable
@@ -576,7 +625,7 @@ module MCollective
       #
       # Should only be called via method_missing
       def fire_and_forget_request(action, args, filter=nil)
-        @ddl.validate_request(action, args) if @ddl
+        @ddl.validate_rpc_request(action, args) if @ddl
 
         req = new_request(action.to_s, args)
 
@@ -693,12 +742,13 @@ module MCollective
 
         message = Message.new(req, nil, {:agent => @agent, :type => :request, :collective => @collective, :filter => opts[:filter], :options => opts})
         message.discovered_hosts = discovered.clone
-        message.type = :direct_request if @force_direct_request
 
         result = []
         respcount = 0
 
         if discovered.size > 0
+          message.type = :direct_request if @force_direct_request
+
           if @progress && !block_given?
             twirl = Progress.new
             @stdout.puts
@@ -766,25 +816,25 @@ module MCollective
           @stats.time_block_execution :start
 
           case block.arity
-          when 1
-            block.call(resp)
-          when 2
-            rpcresp = Result.new(@agent, action, {:sender => resp[:senderid], :statuscode => resp[:body][:statuscode],
-                                   :statusmsg => resp[:body][:statusmsg], :data => resp[:body][:data]})
-            block.call(resp, rpcresp)
+            when 1
+              block.call(resp)
+            when 2
+              rpcresp = Result.new(@agent, action, {:sender => resp[:senderid], :statuscode => resp[:body][:statuscode],
+                                                    :statusmsg => resp[:body][:statusmsg], :data => resp[:body][:data]})
+              block.call(resp, rpcresp)
           end
 
           @stats.time_block_execution :end
         else
           case resp[:body][:statuscode]
-          when 2
-            raise UnknownRPCAction, resp[:body][:statusmsg]
-          when 3
-            raise MissingRPCData, resp[:body][:statusmsg]
-          when 4
-            raise InvalidRPCData, resp[:body][:statusmsg]
-          when 5
-            raise UnknownRPCError, resp[:body][:statusmsg]
+            when 2
+              raise UnknownRPCAction, resp[:body][:statusmsg]
+            when 3
+              raise MissingRPCData, resp[:body][:statusmsg]
+            when 4
+              raise InvalidRPCData, resp[:body][:statusmsg]
+            when 5
+              raise UnknownRPCError, resp[:body][:statusmsg]
           end
         end
       end
