@@ -34,6 +34,8 @@ module MCollective
 
     # Starts the main loop, before calling this you should initialize the MCollective::Config singleton.
     def run
+      Data.load_data_sources
+
       Util.subscribe(Util.make_subscriptions("mcollective", :broadcast))
       Util.subscribe(Util.make_subscriptions("mcollective", :directed)) if @config.direct_addressing
 
@@ -48,10 +50,10 @@ module MCollective
         begin
           request = receive
 
-          if request.agent == "mcollective"
-            controlmsg(request)
-          else
+          unless request.agent == "mcollective"
             agentmsg(request)
+          else
+            Log.error("Received a control message, possibly via 'mco controller' but this has been deprecated")
           end
         rescue SignalException => e
           Log.warn("Exiting after signal: #{e}")
